@@ -3,60 +3,59 @@
 /**
  * @file classes/search/ArticleSearch.inc.php
  *
- * Copyright (c) 2014-2019 Simon Fraser University
- * Copyright (c) 2003-2019 John Willinsky
- * Distributed under the GNU GPL v2. For full terms see the file docs/COPYING.
+ * Copyright (c) 2014-2021 Simon Fraser University
+ * Copyright (c) 2003-2021 John Willinsky
+ * Distributed under the GNU GPL v3. For full terms see the file docs/COPYING.
  *
  * @class ArticleSearch
  * @ingroup search
  * @see ArticleSearchDAO
  *
  * @brief Class for retrieving article search results.
+ *
  */
 
 import('lib.pkp.classes.search.SubmissionSearch');
 
 class ArticleSearch extends SubmissionSearch {
-
 	/**
 	 * See SubmissionSearch::getSparseArray()
 	 */
-	function getSparseArray($unorderedResults, $orderBy, $orderDir, $exclude) {
+	public function getSparseArray($unorderedResults, $orderBy, $orderDir, $exclude) {
 		// Calculate a well-ordered (unique) score.
 		$resultCount = count($unorderedResults);
 		$i = 0;
 		foreach ($unorderedResults as $submissionId => &$data) {
+			// Reference is necessary to permit modification
 			$data['score'] = ($resultCount * $data['count']) + $i++;
-			unset($data);
 		}
 
 		// If we got a primary sort order then apply it and use score as secondary
 		// order only.
 		// NB: We apply order after merging and before paging/formatting. Applying
-		// order before merging (i.e. in ArticleSearchDAO) would require us to
-		// retrieve dependent objects for results being purged later. Doing
-		// everything in a closed SQL is not possible (e.g. for authors). Applying
-		// sort order after paging and formatting is not possible as we have to
-		// order the whole list before slicing it. So this seems to be the most
-		// appropriate place, although we may have to retrieve some objects again
-		// when formatting results.
+		// order before merging would require us to retrieve dependent objects for
+		// results being purged later. Doing everything in a closed SQL is not
+		// possible (e.g. for authors). Applying sort order after paging and
+		// formatting is not possible as we have to order the whole list before
+		// slicing it. So this seems to be the most appropriate place, although we
+		// may have to retrieve some objects again when formatting results.
 		$orderedResults = array();
 		$authorDao = DAORegistry::getDAO('AuthorDAO'); /* @var $authorDao AuthorDAO */
-		$articleDao = DAORegistry::getDAO('ArticleDAO'); /* @var $articleDao ArticleDAO */
-		$journalDao = DAORegistry::getDAO('JournalDAO'); /* @var $journalDao JournalDAO */
-		$journalTitles = array();
+		$submissionDao = DAORegistry::getDAO('SubmissionDAO'); /* @var $submissionDao SubmissionDAO */
+		$contextDao = Application::getContextDAO();
+		$contextTitles = array();
 		if ($orderBy == 'popularityAll' || $orderBy == 'popularityMonth') {
-			$application = Application::getApplication();
+			$application = Application::get();
 			$metricType = $application->getDefaultMetricType();
 			if (is_null($metricType)) {
 				// If no default metric has been found then sort by score...
 				$orderBy = 'score';
 			} else {
-				// Retrieve a metrics report for all articles.
-				$column = STATISTICS_DIMENSION_ARTICLE_ID;
+				// Retrieve a metrics report for all submissions.
+				$column = STATISTICS_DIMENSION_SUBMISSION_ID;
 				$filter = array(
 					STATISTICS_DIMENSION_ASSOC_TYPE => array(ASSOC_TYPE_GALLEY, ASSOC_TYPE_SUBMISSION),
-					STATISTICS_DIMENSION_ARTICLE_ID => array(array_keys($unorderedResults))
+					STATISTICS_DIMENSION_SUBMISSION_ID => array(array_keys($unorderedResults))
 				);
 				if ($orderBy == 'popularityMonth') {
 					$oneMonthAgo = date('Ymd', strtotime('-1 month'));
@@ -77,26 +76,24 @@ class ArticleSearch extends SubmissionSearch {
 
 			switch ($orderBy) {
 				case 'authors':
-					$authors = $authorDao->getBySubmissionId($submissionId);
-					$authorNames = array();
-					foreach ($authors as $author) { /* @var $author Author */
-						$authorNames[] = $author->getFullName(false, true);
-					}
-					$orderKey = implode('; ', $authorNames);
-					unset($authors, $authorNames);
+					$submission = $submissionDao->getById($submissionId);
+					$orderKey = $submission->getAuthorString();
 					break;
 
 				case 'title':
-					$submission = $articleDao->getById($submissionId);
-					$orderKey = $submission->getLocalizedTitle(null, false);
+					$submission = $submissionDao->getById($submissionId);
+					$orderKey = '';
+					if (!empty($submission->getCurrentPublication())) {
+						$orderKey = $submission->getCurrentPublication()->getLocalizedData('title');
+					}
 					break;
 
 				case 'journalTitle':
-					if (!isset($journalTitles[$data['journal_id']])) {
-						$journal = $journalDao->getById($data['journal_id']);
-						$journalTitles[$data['journal_id']] = $journal->getLocalizedName();
+					if (!isset($contextTitles[$data['journal_id']])) {
+						$context = $contextDao->getById($data['journal_id']);
+						$contextTitles[$data['journal_id']] = $context->getLocalizedName();
 					}
-					$orderKey = $journalTitles[$data['journal_id']];
+					$orderKey = $contextTitles[$data['journal_id']];
 					break;
 
 				case 'issuePublicationDate':
@@ -147,7 +144,7 @@ class ArticleSearch extends SubmissionSearch {
 	 * @param $request Request
 	 * @return array All search filters (empty and active)
 	 */
-	function getSearchFilters($request) {
+	public function getSearchFilters($request) {
 		$searchFilters = array(
 			'query' => $request->getUserVar('query'),
 			'searchJournal' => $request->getUserVar('searchJournal'),
@@ -180,24 +177,24 @@ class ArticleSearch extends SubmissionSearch {
 		$toDate = $request->getUserDateVar('dateTo', 32, 12, null, 23, 59, 59);
 		$searchFilters['toDate'] = (is_null($toDate) ? null : date('Y-m-d H:i:s', $toDate));
 
-		// Instantiate the journal.
-		$journal = $request->getJournal();
-		$siteSearch = !((boolean)$journal);
+		// Instantiate the context.
+		$context = $request->getContext();
+		$siteSearch = !((boolean)$context);
 		if ($siteSearch) {
-			$journalDao = DAORegistry::getDAO('JournalDAO'); /* @var $journalDao JournalDAO */
+			$contextDao = Application::getContextDAO();
 			if (!empty($searchFilters['searchJournal'])) {
-				$journal = $journalDao->getById($searchFilters['searchJournal']);
+				$context = $contextDao->getById($searchFilters['searchJournal']);
 			} elseif (array_key_exists('journalTitle', $request->getUserVars())) {
-				$journals = $journalDao->getTitles(false);
-				while ($journal = $journals->next()) {
+				$contexts = $contextDao->getAll(true);
+				while ($context = $contexts->next()) {
 					if (in_array(
 						$request->getUserVar('journalTitle'),
-						(array) $journal->getTitle(null)
+						(array) $context->getTitle(null)
 					)) break;
 				}
 			}
 		}
-		$searchFilters['searchJournal'] = $journal;
+		$searchFilters['searchJournal'] = $context;
 		$searchFilters['siteSearch'] = $siteSearch;
 
 		return $searchFilters;
@@ -209,7 +206,7 @@ class ArticleSearch extends SubmissionSearch {
 	 *  ArticleSearch::getSearchFilters()
 	 * @return array Keyword array as required by SubmissionSearch::retrieveResults()
 	 */
-	function getKeywordsFromSearchFilters($searchFilters) {
+	public function getKeywordsFromSearchFilters($searchFilters) {
 		$indexFieldMap = $this->getIndexFieldMap();
 		$indexFieldMap[SUBMISSION_SEARCH_INDEX_TERMS] = 'indexTerms';
 		$keywords = array();
@@ -229,64 +226,63 @@ class ArticleSearch extends SubmissionSearch {
 	 *
 	 * @param $results array
 	 * @param $user User optional (if availability information is desired)
-	 * @return array An array with the articles, published articles,
+	 * @return array An array with the articles, published submissions,
 	 *  issue, journal, section and the issue availability.
 	 */
-	function formatResults($results, $user = null) {
-		$articleDao = DAORegistry::getDAO('ArticleDAO');
-		$publishedArticleDao = DAORegistry::getDAO('PublishedArticleDAO');
-		$issueDao = DAORegistry::getDAO('IssueDAO');
-		$journalDao = DAORegistry::getDAO('JournalDAO');
-		$sectionDao = DAORegistry::getDAO('SectionDAO');
+	public function formatResults($results, $user = null) {
+		$issueDao = DAORegistry::getDAO('IssueDAO'); /* @var $issueDao IssueDAO */
+		$contextDao = Application::getContextDAO();
+		$sectionDao = DAORegistry::getDAO('SectionDAO'); /* @var $sectionDao SectionDAO */
 
-		$publishedArticleCache = array();
+		$publishedSubmissionCache = array();
 		$articleCache = array();
 		$issueCache = array();
 		$issueAvailabilityCache = array();
-		$journalCache = array();
+		$contextCache = array();
 		$sectionCache = array();
 
 		$returner = array();
 		foreach ($results as $articleId) {
 			// Get the article, storing in cache if necessary.
 			if (!isset($articleCache[$articleId])) {
-				$publishedArticleCache[$articleId] = $publishedArticleDao->getByArticleId($articleId);
-				$articleCache[$articleId] = $articleDao->getById($articleId);
+				$submission = Services::get('submission')->get($articleId);
+				$publishedSubmissionCache[$articleId] = $submission;
+				$articleCache[$articleId] = $submission;
 			}
 			$article = $articleCache[$articleId];
-			$publishedArticle = $publishedArticleCache[$articleId];
+			$publishedSubmission = $publishedSubmissionCache[$articleId];
 
-			if ($publishedArticle && $article) {
+			if ($publishedSubmission && $article) {
 				$sectionId = $article->getSectionId();
 				if (!isset($sectionCache[$sectionId])) {
 					$sectionCache[$sectionId] = $sectionDao->getById($sectionId);
 				}
 
-				// Get the journal, storing in cache if necessary.
-				$journalId = $article->getJournalId();
-				if (!isset($journalCache[$journalId])) {
-					$journalCache[$journalId] = $journalDao->getById($journalId);
+				// Get the context, storing in cache if necessary.
+				$contextId = $article->getData('contextId');
+				if (!isset($contextCache[$contextId])) {
+					$contextCache[$contextId] = $contextDao->getById($contextId);
 				}
 
 				// Get the issue, storing in cache if necessary.
-				$issueId = $publishedArticle->getIssueId();
+				$issueId = $publishedSubmission->getCurrentPublication()->getData('issueId');
 				if (!isset($issueCache[$issueId])) {
 					$issue = $issueDao->getById($issueId);
 					$issueCache[$issueId] = $issue;
 					import('classes.issue.IssueAction');
 					$issueAction = new IssueAction();
-					$issueAvailabilityCache[$issueId] = !$issueAction->subscriptionRequired($issue, $journalCache[$journalId]) || $issueAction->subscribedUser($user, $journalCache[$journalId], $issueId, $articleId) || $issueAction->subscribedDomain(Application::getRequest(), $journalCache[$journalId], $issueId, $articleId);
+					$issueAvailabilityCache[$issueId] = !$issueAction->subscriptionRequired($issue, $contextCache[$contextId]) || $issueAction->subscribedUser($user, $contextCache[$contextId], $issueId, $articleId) || $issueAction->subscribedDomain(Application::get()->getRequest(), $contextCache[$contextId], $issueId, $articleId);
 				}
 
 				// Only display articles from published issues.
-				if (!$issueCache[$issueId]->getPublished()) continue;
+				if (!isset($issueCache[$issueId]) || !$issueCache[$issueId]->getPublished()) continue;
 
 				// Store the retrieved objects in the result array.
 				$returner[] = array(
 					'article' => $article,
-					'publishedArticle' => $publishedArticleCache[$articleId],
+					'publishedSubmission' => $publishedSubmissionCache[$articleId],
 					'issue' => $issueCache[$issueId],
-					'journal' => $journalCache[$journalId],
+					'journal' => $contextCache[$contextId],
 					'issueAvailable' => $issueAvailabilityCache[$issueId],
 					'section' => $sectionCache[$sectionId]
 				);
@@ -301,7 +297,7 @@ class ArticleSearch extends SubmissionSearch {
 	 * @return null|array An array of string keywords or null
 	 * if some kind of error occurred.
 	 */
-	function getSimilarityTerms($submissionId) {
+	public function getSimilarityTerms($submissionId) {
 		// Check whether a search plugin provides terms for a similarity search.
 		$searchTerms = array();
 		$result = HookRegistry::call('ArticleSearch::getSimilarityTerms', array($submissionId, &$searchTerms));
@@ -310,21 +306,21 @@ class ArticleSearch extends SubmissionSearch {
 		// of the submission for a similarity search.
 		if ($result === false) {
 			// Retrieve the article.
-			$publishedArticleDao = DAORegistry::getDAO('PublishedArticleDAO'); /* @var $publishedArticleDao PublishedArticleDAO */
-			$article = $publishedArticleDao->getByArticleId($submissionId);
-			if (is_a($article, 'PublishedArticle')) {
+			$article = Services::get('submission')->get($submissionId);
+			if ($article->getData('status') === STATUS_PUBLISHED) {
 				// Retrieve keywords (if any).
-				$searchTerms = $article->getLocalizedSubject();
-				// Tokenize keywords.
-				$searchTerms = trim(preg_replace('/\s+/', ' ', strtr($searchTerms, ',;', ' ')));
-				if (!empty($searchTerms)) $searchTerms = explode(' ', $searchTerms);
+				$submissionSubjectDao = DAORegistry::getDAO('SubmissionKeywordDAO'); /* @var $submissionSubjectDao SubmissionKeywordDAO */
+				$allSearchTerms = array_filter($submissionSubjectDao->getKeywords($article->getCurrentPublication()->getId(), array(AppLocale::getLocale(), $article->getLocale(), AppLocale::getPrimaryLocale())));
+				foreach ($allSearchTerms as $locale => $localeSearchTerms) {
+					$searchTerms += $localeSearchTerms;
+				}
 			}
 		}
 
 		return $searchTerms;
 	}
 
-	function getIndexFieldMap() {
+	public function getIndexFieldMap() {
 		return array(
 			SUBMISSION_SEARCH_AUTHOR => 'authors',
 			SUBMISSION_SEARCH_TITLE => 'title',
@@ -332,6 +328,7 @@ class ArticleSearch extends SubmissionSearch {
 			SUBMISSION_SEARCH_GALLEY_FILE => 'galleyFullText',
 			SUBMISSION_SEARCH_DISCIPLINE => 'discipline',
 			SUBMISSION_SEARCH_SUBJECT => 'subject',
+			SUBMISSION_SEARCH_KEYWORD => 'keyword',
 			SUBMISSION_SEARCH_TYPE => 'type',
 			SUBMISSION_SEARCH_COVERAGE => 'coverage'
 		);
@@ -340,7 +337,7 @@ class ArticleSearch extends SubmissionSearch {
 	/**
 	 * See SubmissionSearch::getResultSetOrderingOptions()
 	 */
-	function getResultSetOrderingOptions($request) {
+	public function getResultSetOrderingOptions($request) {
 		$resultSetOrderingOptions = array(
 			'score' => __('search.results.orderBy.relevance'),
 			'authors' => __('search.results.orderBy.author'),
@@ -350,7 +347,7 @@ class ArticleSearch extends SubmissionSearch {
 		);
 
 		// Only show the "popularity" options if we have a default metric.
-		$application = Application::getApplication();
+		$application = Application::get();
 		$metricType = $application->getDefaultMetricType();
 		if (!is_null($metricType)) {
 			$resultSetOrderingOptions['popularityAll'] = __('search.results.orderBy.popularityAll');
@@ -358,15 +355,15 @@ class ArticleSearch extends SubmissionSearch {
 		}
 
 		// Only show the "journal title" option if we have several journals.
-		$journal = $request->getContext();
-		if (!is_a($journal, 'Journal')) {
+		$context = $request->getContext();
+		if (!$context) {
 			$resultSetOrderingOptions['journalTitle'] = __('search.results.orderBy.journal');
 		}
 
 		// Let plugins mangle the search ordering options.
 		HookRegistry::call(
 			'SubmissionSearch::getResultSetOrderingOptions',
-			array($journal, &$resultSetOrderingOptions)
+			array($context, &$resultSetOrderingOptions)
 		);
 
 		return $resultSetOrderingOptions;
@@ -375,7 +372,7 @@ class ArticleSearch extends SubmissionSearch {
 	/**
 	 * See SubmissionSearch::getDefaultOrderDir()
 	 */
-	function getDefaultOrderDir($orderBy) {
+	public function getDefaultOrderDir($orderBy) {
 		$orderDir = 'asc';
 		if (in_array($orderBy, array('score', 'publicationDate', 'issuePublicationDate', 'popularityAll', 'popularityMonth'))) {
 			$orderDir = 'desc';
