@@ -3,9 +3,9 @@
 /**
  * @file classes/plugins/ThemePlugin.inc.php
  *
- * Copyright (c) 2014-2016 Simon Fraser University Library
- * Copyright (c) 2003-2016 John Willinsky
- * Distributed under the GNU GPL v2. For full terms see the file docs/COPYING.
+ * Copyright (c) 2014-2021 Simon Fraser University
+ * Copyright (c) 2003-2021 John Willinsky
+ * Distributed under the GNU GPL v3. For full terms see the file docs/COPYING.
  *
  * @class ThemePlugin
  * @ingroup plugins
@@ -42,6 +42,13 @@ abstract class ThemePlugin extends LazyLoadPlugin {
 	 */
 	public $options = array();
 
+	/**
+	 * Theme-specific navigation menu areas
+	 *
+	 * @var $menuAreas array;
+	 */
+	public $menuAreas = array();
+
 	/**
 	 * Parent theme (optional)
 	 *
@@ -55,22 +62,15 @@ abstract class ThemePlugin extends LazyLoadPlugin {
 	 * A null value indicates that no lookup has occured. If no options are set,
 	 * the lookup will assign an empty array.
 	 *
-	 * @var $optionValues null|array;
+	 * @var $_optionValues null|array;
 	 */
-	private $_optionValues = null;
-
-	/**
-	 * Constructor
-	 */
-	function ThemePlugin() {
-		parent::LazyLoadPlugin();
-	}
+	protected $_optionValues = null;
 
 	/**
 	 * @copydoc Plugin::register
 	 */
-	function register($category, $path) {
-		if (!parent::register($category, $path)) return false;
+	function register($category, $path, $mainContextId = null) {
+		if (!parent::register($category, $path, $mainContextId)) return false;
 
 		// Don't perform any futher operations if theme is not currently active
 		if (!$this->isActive()) {
@@ -83,12 +83,8 @@ abstract class ThemePlugin extends LazyLoadPlugin {
 		HookRegistry::register('PluginRegistry::categoryLoaded::themes', array($this, 'themeRegistered'));
 		HookRegistry::register('PluginRegistry::categoryLoaded::themes', array($this, 'initAfter'));
 
-		// Save any theme options displayed on the appearance and site settings
-		// forms
-		HookRegistry::register('appearanceform::execute', array($this, 'saveOptionsForm'));
-		HookRegistry::register('appearanceform::readuservars', array($this, 'readOptionsFormUserVars'));
-		HookRegistry::register('sitesetupform::execute', array($this, 'saveOptionsForm'));
-		HookRegistry::register('sitesetupform::readuservars', array($this, 'readOptionsFormUserVars'));
+		// Allow themes to override plugin template files
+		HookRegistry::register('TemplateResource::getFilename', array($this, '_overridePluginTemplates'));
 
 		return true;
 	}
@@ -142,13 +138,13 @@ abstract class ThemePlugin extends LazyLoadPlugin {
 	 */
 	public function isActive() {
 		if (defined('SESSION_DISABLE_INIT')) return false;
-		$request = $this->getRequest();
+		$request = Application::get()->getRequest();
 		$context = $request->getContext();
 		if (is_a($context, 'Context')) {
-			$activeTheme = $context->getSetting('themePluginPath');
+			$activeTheme = $context->getData('themePluginPath');
 		} else {
 			$site = $request->getSite();
-			$activeTheme = $site->getSetting('themePluginPath');
+			$activeTheme = $site->getData('themePluginPath');
 		}
 
 		return $activeTheme == basename($this->getPluginPath());
@@ -230,7 +226,23 @@ abstract class ThemePlugin extends LazyLoadPlugin {
 			$args['style'] = substr($args['style'], (strlen(LESS_FILENAME_SUFFIX) * -1)) == LESS_FILENAME_SUFFIX ? $this->_getBaseDir($args['style']) : $this->_getBaseUrl($args['style']);
 		}
 
-		$style = array_merge($style, $args);
+		$style = array_merge_recursive($style, $args);
+	}
+
+	/**
+	 * Remove a registered stylesheet
+	 *
+	 * @param $name string The name of the stylesheet to remove
+	 * @return bool Whether or not the stylesheet was found and removed.
+	 */
+	public function removeStyle($name) {
+
+		if (isset($this->styles[$name])) {
+			unset($this->styles[$name]);
+			return true;
+		}
+
+		return $this->parent ? $this->parent->removeStyle($name) : false;
 	}
 
 	/**
@@ -307,6 +319,22 @@ abstract class ThemePlugin extends LazyLoadPlugin {
 		$script = array_merge( $script, $args );
 	}
 
+	/**
+	 * Remove a registered script
+	 *
+	 * @param $name string The name of the script to remove
+	 * @return bool Whether or not the stylesheet was found and removed.
+	 */
+	public function removeScript($name) {
+
+		if (isset($this->scripts[$name])) {
+			unset($this->scripts[$name]);
+			return true;
+		}
+
+		return $this->parent ? $this->parent->removeScript($name) : false;
+	}
+
 	/**
 	 * Get a script from this theme or any parent theme
 	 *
@@ -337,8 +365,7 @@ abstract class ThemePlugin extends LazyLoadPlugin {
 	 * colour and typography selectors.
 	 *
 	 * @param $name string Unique name for this setting
-	 * @param $type string A pre-registered type of setting. Supported values:
-	 *   text|colour|select. Default: `text`
+	 * @param $type string One of the Field* class names
 	 * @param $args array Optional parameters defining this setting. Some setting
 	 *   types may accept or require additional arguments.
 	 *  `label` string Locale key for a label for this field.
@@ -351,27 +378,73 @@ abstract class ThemePlugin extends LazyLoadPlugin {
 			return;
 		}
 
-		$this->options[$name] = array_merge(
-			array('type' => $type),
-			$args
-		);
+		// Convert theme option types from before v3.2
+		if (in_array($type, ['text', 'colour', 'radio'])) {
+			if (isset($args['label'])) {
+				$args['label'] = __($args['label']);
+			}
+			if (isset($args['description'])) {
+				$args['description'] = __($args['description']);
+			}
+			switch ($type) {
+				case 'text':
+					$type = 'FieldText';
+					break;
+				case 'colour':
+					$type = 'FieldColor';
+					break;
+				case 'radio':
+					$type = 'FieldOptions';
+					$args['type'] = 'radio';
+					if (!empty($args['options'])) {
+						$options = [];
+						foreach ($args['options'] as $optionValue => $optionLabel) {
+							$options[] = ['value' => $optionValue, 'label' => __($optionLabel)];
+						}
+						$args['options'] = $options;
+					}
+					break;
+			}
+		}
+
+		$class = 'PKP\components\forms\\' . $type;
+		try {
+			$this->options[$name] = new $class($name, $args);
+		} catch (Exception $e) {
+			$class = 'APP\components\forms\\' . $type;
+			try {
+				$this->options[$name] = new $class($name, $args);
+			} catch (Exception $e) {
+				throw new Exception(sprintf(
+					'The %s class was not found for the theme option, %s,  defined by %s or one of its parent themes.',
+					$type,
+					$name,
+					$this->getDisplayName()
+				));
+			}
+		}
 	}
 
 	/**
 	 * Get the value of an option or default if the option is not set
 	 *
-	 * @param $name The name of the option value to retrieve
+	 * @param $name string The name of the option value to retrieve
 	 * @return mixed The value of the option. Will return a default if set in
-	 *  the option config.
+	 *  the option config. False if no option exists. Null if no value or default
+	 *  exists.
 	 */
 	public function getOption($name) {
 
+		// Check if this is a valid option
+		if (!isset($this->options[$name])) {
+			return $this->parent ? $this->parent->getOption($name) : false;
+		}
+
 		// Retrieve option values if they haven't been loaded yet
 		if (is_null($this->_optionValues)) {
-			$pluginSettingsDAO = DAORegistry::getDAO('PluginSettingsDAO');
-			$context = Request::getContext();
-			$contextId = $context ? $context->getId() : 0;
-			$this->_optionValues = $pluginSettingsDAO->getPluginSettings($contextId, $this->getName());
+			$context = Application::get()->getRequest()->getContext();
+			$contextId = $context ? $context->getId() : CONTEXT_ID_NONE;
+			$this->_optionValues = $this->getOptionValues($contextId);
 		}
 
 		if (isset($this->_optionValues[$name])) {
@@ -379,8 +452,12 @@ abstract class ThemePlugin extends LazyLoadPlugin {
 		}
 
 		// Return a default if no value is set
-		$option = $this->getOptionConfig($name);
-		return $option && isset($option['default']) ? $option['default'] : null;
+		if (isset($this->options[$name])) {
+			$option = $this->options[$name];
+		} elseif ($this->parent) {
+			$option = $this->parent->getOption($name);
+		}
+		return isset($option->default) ? $option->default : null;
 	}
 
 	/**
@@ -422,32 +499,109 @@ abstract class ThemePlugin extends LazyLoadPlugin {
 		);
 	}
 
+	/**
+	 * Modify option configuration settings
+	 *
+	 * @deprecated Unnecessary since 3.2 because options are stored as objects,
+	 *  so changes can be made directly (via reference) and args don't need to be
+	 *  manually merged
+	 * @param $name The name of the option config to retrieve
+	 * @param $args The new configuration settings for this option
+	 * @return bool Whether the option was found and the config was updated.
+	 */
+	public function modifyOptionsConfig($name, $args = array()) {
+		$option = $this->getOption($name);
+		foreach ($args as $key => $value) {
+			if (property_exists($option, $key)) {
+				$option->{$key} = $value;
+			}
+		}
+	}
+
+	/**
+	 * Remove an option
+	 *
+	 * @param $name The name of the option to remove
+	 * @return bool Whether the option was found and removed
+	 */
+	public function removeOption($name) {
+
+		if (isset($this->options[$name])) {
+			unset($this->options[$name]);
+			return true;
+		}
+
+		return $this->parent ? $this->parent->removeOption($name) : false;
+	}
+
 	/**
 	 * Get all option values
 	 *
 	 * This retrieves a single array containing option values for this theme
 	 * and any parent themes.
 	 *
+	 * @param $contextId int
 	 * @return array
 	 */
-	public function getOptionValues() {
+	public function getOptionValues($contextId) {
 
 		$pluginSettingsDAO = DAORegistry::getDAO('PluginSettingsDAO');
 
-		$context = Request::getContext();
-		$contextId = empty($context) ? 0 : $context->getId();
+		$return = [];
 		$values = $pluginSettingsDAO->getPluginSettings($contextId, $this->getName());
+		foreach ($this->options as $optionName => $optionConfig) {
+			$value = isset($values[$optionName]) ? $values[$optionName] : null;
+			// Convert values stored in the db to the type of the default value
+			if (!is_null($optionConfig->default)) {
+				switch (gettype($optionConfig->default)) {
+					case 'boolean':
+						$value = !$value || $value === 'false' ? false : true;
+						break;
+					case 'integer':
+						$value = (int) $value;
+						break;
+					case 'array':
+						$value = $value === null ? [] : unserialize($value);
+						break;
+				}
+			}
+			$return[$optionName] = $value;
+		}
 
 		if (!$this->parent) {
-			return $values;
+			return $return;
 		}
 
 		return array_merge(
-			$this->parent->getOptionValues(),
-			$values
+			$this->parent->getOptionValues($contextId),
+			$return
 		);
 	}
 
+	/**
+	 * Overwrite this function to perform any validation on options before they
+	 * are saved
+	 *
+	 * If this is a child theme, you must call $this->parent->validateOptions() to
+	 * perform any validation defined on the parent theme.
+	 *
+	 * @param $options array Key/value list of options to validate
+	 * @param $themePluginPath string The theme these options are for
+	 * @param $contextId int The context these theme options are for, or
+	 *  CONTEXT_ID_NONE for the site-wide settings.
+	 * @param $request Request
+	 * @return array List of errors with option name as the key and the value as
+	 *  an array of error messages. Example:
+	 *  [
+	 *    'color' => [
+	 *      'This color is too dark for this area and some people will not be able to read it.',
+	 *    ]
+	 *  ]
+	 */
+	public function validateOptions($options, $themePluginPath, $contextId, $request) {
+		return [];
+	}
+
 	/**
 	 * Sanitize and save a theme option
 	 *
@@ -464,87 +618,66 @@ abstract class ThemePlugin extends LazyLoadPlugin {
 			return $this->parent ? $this->parent->saveOption($name, $value, $contextId) : false;
 		}
 
-		$type = '';
-		switch ($option['type']) {
-			case 'text' :
-			case 'select' :
-			case 'colour' :
-				$type = 'text';
-				break;
-		}
-
 		if (is_null($contextId)) {
-			$context = Request::getContext();
+			$context = Application::get()->getRequest()->getContext();
 			$contextId = $context->getId();
 		}
 
-		$this->updateSetting($contextId, $name, $value, $type);
+		$pluginSettingsDao = DAORegistry::getDAO('PluginSettingsDAO'); /* @var $pluginSettingsDao PluginSettingsDAO */
+
+		// Remove setting row for empty string values (but not all falsey values)
+		if ($value === '') {
+			$pluginSettingsDao->deleteSetting($contextId, $this->getName(), $name);
+		} else {
+			$type = $pluginSettingsDao->getType($value);
+			$value = $pluginSettingsDao->convertToDb($value, $type);
+			$this->updateSetting($contextId, $name, $value, $type);
+		}
 
-		// Clear the template cache so that new settings can take effect
-		$templateMgr = TemplateManager::getManager($this->getRequest());
-		$templateMgr->clearTemplateCache();
-		$templateMgr->clearCssCache();
 	}
 
 	/**
-	 * Save options in any form
+	 * Register a navigation menu area for this theme
 	 *
-	 * This helper function allows you to save theme options attached to any
-	 * form by hooking into the form's execute function.
-	 *
-	 * @see Form::execute()
-	 * @param $hookName string
-	 * @param $args array Arguments passed via the hook
-	 *  `form` Form The form object from which option values can be retrieved.
-	 *  `request` Request
+	 * @param $menuAreas string|array One or more menu area names
 	 */
-	public function saveOptionsForm($hookName, $args) {
+	public function addMenuArea($menuAreas) {
 
-		$form = $args[0];
-
-		$options = $this->getOptionsConfig();
-
-		// Ensure theme options from the site-wide settings form are applied
-		// to the site-wide context
-		if ($hookName == 'sitesetupform::execute') {
-			$contextId = 0;
+		if (!is_array($menuAreas)) {
+			$menuAreas = array($menuAreas);
 		}
 
-		foreach ($options as $optionName => $optionArgs) {
-			$value = $form->getData(THEME_OPTION_PREFIX . $optionName);
-			if ($value === null) {
-				continue;
-			}
-			if (isset($contextId)) {
-				$this->saveOption($optionName, $value, $contextId);
-			} else {
-				$this->saveOption($optionName, $value);
-			}
-		}
+		$this->menuAreas = array_merge($this->menuAreas, $menuAreas);
 	}
 
 	/**
-	 * Retrieve user-entered values for options from any form
+	 * Remove a registered navigation menu area
 	 *
-	 * This helper function allows you to hook into any form to add theme option
-	 * values to the form's user input data.
-	 *
-	 * @see Form::readUserVar()
-	 * @param $hookName string
-	 * @param $args array Arguments passed via the hook
-	 *  `form` Form The form object from which option values can be retrieved.
-	 *  `vars` Array Key/value store of the user vars read by the form
+	 * @param $menuArea string The menu area to remove
+	 * @return bool Whether or not the menuArea was found and removed.
 	 */
-	public function readOptionsFormUserVars($hookName, $args) {
+	public function removeMenuArea($menuArea) {
 
-		$form = $args[0];
-
-		$options = $this->getOptionsConfig();
-
-		foreach ($options as $optionName => $optionArgs) {
-			$fullOptionName = THEME_OPTION_PREFIX . $optionName;
-			$form->setData($fullOptionName, Request::getUserVar($fullOptionName));
+		$index = array_search($menuArea, $this->menuAreas);
+		if ($index !== false) {
+			array_splice($this->menuAreas, $index, 1);
+			return true;
 		}
+
+		return $this->parent ? $this->parent->removeMenuArea($menuArea) : false;
+	}
+
+	/**
+	 * Get all menu areas registered by this theme and any parents
+	 *
+	 * @param $existingAreas array Any existing menu areas from child themes
+	 * @return array All menua reas
+	 */
+	public function getMenuAreas($existingAreas = array()) {
+
+		$existingAreas = array_unique(array_merge($this->menuAreas, $existingAreas));
+
+		return $this->parent ? $this->parent->getMenuAreas($existingAreas) : $existingAreas;
 	}
 
 	/**
@@ -577,12 +710,9 @@ abstract class ThemePlugin extends LazyLoadPlugin {
 		}
 
 		// Register this theme's template directory
-		$request = $this->getRequest();
+		$request = Application::get()->getRequest();
 		$templateManager = TemplateManager::getManager($request);
-		array_unshift(
-			$templateManager->template_dir,
-			$this->_getBaseDir('templates')
-		);
+		$templateManager->addTemplateDir($this->_getBaseDir('templates'));
 	}
 
 	/**
@@ -598,7 +728,7 @@ abstract class ThemePlugin extends LazyLoadPlugin {
 			$this->parent->_registerStyles();
 		}
 
-		$request = $this->getRequest();
+		$request = Application::get()->getRequest();
 		$dispatcher = $request->getDispatcher();
 		$templateManager = TemplateManager::getManager($request);
 
@@ -644,7 +774,7 @@ abstract class ThemePlugin extends LazyLoadPlugin {
 			$this->parent->_registerScripts();
 		}
 
-		$request = $this->getRequest();
+		$request = Application::get()->getRequest();
 		$templateManager = TemplateManager::getManager($request);
 
 		foreach($this->scripts as $name => $data) {
@@ -665,7 +795,7 @@ abstract class ThemePlugin extends LazyLoadPlugin {
 	 * @return string
 	 */
 	public function _getBaseUrl($path = '') {
-		$request = $this->getRequest();
+		$request = Application::get()->getRequest();
 		$path = empty($path) ? '' : DIRECTORY_SEPARATOR . $path;
 		return $request->getBaseUrl() . DIRECTORY_SEPARATOR . $this->getPluginPath() . $path;
 	}
@@ -694,11 +824,11 @@ abstract class ThemePlugin extends LazyLoadPlugin {
 	 *
 	 * @since 0.1
 	 */
-	function isColourDark( $colour, $limit = 130 ) {
-		$colour = str_replace( '#', '', $colour );
-		$r = hexdec( substr( $colour, 0, 2 ) );
-		$g = hexdec( substr( $colour, 2, 2 ) );
-		$b = hexdec( substr( $colour, 4, 2 ) );
+	public function isColourDark($color, $limit = 130) {
+		$color = str_replace('#', '', $color);
+		$r = hexdec(substr($color, 0, 2));
+		$g = hexdec(substr($color, 2, 2));
+		$b = hexdec(substr($color, 4, 2));
 		$contrast = sqrt(
 			$r * $r * .241 +
 			$g * $g * .691 +
@@ -707,5 +837,3 @@ abstract class ThemePlugin extends LazyLoadPlugin {
 		return $contrast < $limit;
 	}
 }
-
-?>
