@@ -4,9 +4,9 @@
 /**
  * @file js/controllers/SiteHandler.js
  *
- * Copyright (c) 2014-2016 Simon Fraser University Library
- * Copyright (c) 2000-2016 John Willinsky
- * Distributed under the GNU GPL v2. For full terms see the file docs/COPYING.
+ * Copyright (c) 2014-2021 Simon Fraser University
+ * Copyright (c) 2000-2021 John Willinsky
+ * Distributed under the GNU GPL v3. For full terms see the file docs/COPYING.
  *
  * @class SiteHandler
  * @ingroup js_controllers
@@ -24,9 +24,6 @@
 	 * @param {jQueryObject} $widgetWrapper An HTML element that this handle will
 	 * be attached to.
 	 * @param {{
-	 *   toggleHelpUrl: string,
-	 *   toggleHelpOffText: string,
-	 *   toggleHelpOnText: string,
 	 *   fetchNotificationUrl: string,
 	 *   requestOptions: Object
 	 *   }} options Handler options.
@@ -41,7 +38,7 @@
 
 		this.bind('redirectRequested', this.redirectToUrl);
 		this.bind('notifyUser', this.fetchNotificationHandler_);
-		this.bind('updateHeader', this.updateHeaderHandler_);
+		this.bind('reloadTab', this.reloadTabHandler_);
 		this.bind('callWhenClickOutside', this.callWhenClickOutsideHandler_);
 		this.bind('mousedown', this.mouseDownHandler_);
 
@@ -130,36 +127,73 @@
 			function() {
 
 		if (typeof tinyMCE !== 'undefined') {
-			tinyMCE.PluginManager.load('jbimages', $.pkp.app.baseUrl +
-					'/plugins/generic/tinymce/plugins/justboil.me/plugin.js');
 			tinyMCE.PluginManager.load('pkpTags', $.pkp.app.baseUrl +
 					'/plugins/generic/tinymce/plugins/pkpTags/plugin.js');
+			tinyMCE.PluginManager.load('pkpwordcount', $.pkp.app.baseUrl +
+					'/plugins/generic/tinymce/plugins/pkpWordcount/plugin.js');
 
-			var tinymceParams, tinymceParamDefaults = {
+
+			var contentCSS = $.pkp.app.tinyMceContentCSS,
+					tinymceParams,
+					tinymceParamDefaults;
+
+			tinymceParamDefaults = {
 				width: '100%',
 				resize: 'both',
 				entity_encoding: 'raw',
-				plugins: 'paste,fullscreen,link,code,-jbimages,-pkpTags,noneditable',
+				plugins: 'paste,fullscreen,link,lists,code,' +
+						'image,-pkpTags,noneditable',
 				convert_urls: false,
 				forced_root_block: 'p',
 				paste_auto_cleanup_on_paste: true,
 				apply_source_formatting: false,
 				theme: 'modern',
 				toolbar: 'copy paste | bold italic underline | link unlink ' +
-						'code fullscreen | jbimages | pkpTags',
+						'code fullscreen | image | pkpTags',
 				richToolbar: 'copy paste | bold italic underline | bullist numlist | ' +
 						'superscript subscript | link unlink code fullscreen | ' +
-						'jbimages | pkpTags',
+						'image | pkpTags',
 				statusbar: false,
-				content_css: $.pkp.app.baseUrl +
-						'/plugins/generic/tinymce/styles/content.css'
+				content_css: contentCSS
 			};
 
+			// Support image uploads
+			if (typeof $.pkp.plugins.generic.tinymceplugin !== 'undefined' &&
+					typeof $.pkp.plugins.generic.tinymceplugin.uploadUrl !== 'undefined') {
+				tinymceParamDefaults.paste_data_images = true;
+				tinymceParamDefaults.relative_urls = false;
+				tinymceParamDefaults.remove_script_host = false;
+				// https://www.tiny.cloud/docs/general-configuration-guide/upload-images/
+				tinymceParamDefaults.images_upload_handler = function(
+						blobInfo, success, failure) {
+
+					/*global FormData: false */
+					var data = new FormData();
+					data.append('file', blobInfo.blob(), blobInfo.filename());
+					$.ajax({
+						method: 'POST',
+						url: $.pkp.plugins.generic.tinymceplugin.uploadUrl,
+						data: data,
+						processData: false,
+						contentType: false,
+						headers: {
+							'X-Csrf-Token': pkp.currentUser.csrfToken
+						},
+						success: function(r) {
+							success(r.url);
+						},
+						error: function(r) {
+							failure(r.responseJSON.errorMessage);
+						}
+					});
+				};
+			}
+
 			// Allow default params to be overridden
-			if (typeof $.pkp.plugins.tinymceplugin !== 'undefined' &&
-					typeof $.pkp.plugins.tinymceplugin.tinymceParams) {
+			if (typeof $.pkp.plugins.generic.tinymceplugin !== 'undefined' &&
+					typeof $.pkp.plugins.generic.tinymceplugin.tinymceParams) {
 				tinymceParams = $.extend({}, tinymceParamDefaults,
-						$.pkp.plugins.tinymceplugin.tinymceParams);
+						$.pkp.plugins.generic.tinymceplugin.tinymceParams);
 			} else {
 				tinymceParams = $.extend({}, tinymceParamDefaults);
 			}
@@ -184,7 +218,8 @@
 	$.pkp.controllers.SiteHandler.prototype.triggerTinyMCEInitialized =
 			function(tinyMCEObject) {
 
-		var $inputElement = $('#' + tinyMCEObject.id);
+		var $inputElement = $('#' +
+				$.pkp.classes.Helper.escapeJQuerySelector(tinyMCEObject.id));
 		$inputElement.trigger('tinyMCEInitialized', [tinyMCEObject]);
 	};
 
@@ -196,13 +231,25 @@
 	 */
 	$.pkp.controllers.SiteHandler.prototype.triggerTinyMCESetup =
 			function(tinyMCEObject) {
-		var target = $('#' + tinyMCEObject.id), height;
+		var target = $('#' +
+				$.pkp.classes.Helper.escapeJQuerySelector(tinyMCEObject.id)),
+				height;
 
 		// For read-only controls, set up TinyMCE read-only mode.
 		if (target.attr('readonly')) {
 			tinyMCEObject.settings.readonly = true;
 		}
 
+		if (target.attr('wordCount') && target.attr('wordCount') > 0) {
+			tinyMCEObject.settings.plugins =
+					tinyMCEObject.settings.plugins + ',pkpwordcount';
+			tinyMCEObject.settings.statusbar = true;
+		}
+
+		if (target.attr('dir')) {
+			tinyMCEObject.settings.directionality = target.attr('dir');
+		}
+
 		// Set height based on textarea rows
 		height = target.attr('rows') || 10; // default: 10
 		height *= 20; // 20 pixels per row
@@ -222,8 +269,8 @@
 			}
 
 			// Create placeholder element
-			$placeholder = /** @type {jQueryObject} */ ($('<span></span>')
-					.html(/** @type {string} */ (placeholderText)));
+			$placeholder = $('<span></span>');
+			$placeholder.html(/** @type {string} */ (placeholderText));
 			$placeholder.addClass('mcePlaceholder');
 			$placeholder.attr('id', 'mcePlaceholder-' + tinyMCEObject.id);
 
@@ -254,13 +301,14 @@
 
 		tinyMCEObject.on('BeforeSetContent', function(e) {
 			var variablesParsed = $.pkp.classes.TinyMCEHelper.prototype.getVariableMap(
-					'#' + tinyMCEObject.id);
+					'#' + $.pkp.classes.Helper.escapeJQuerySelector(tinyMCEObject.id));
 
 			e.content = e.content.replace(
 					/\{\$([a-zA-Z]+)\}(?![^<]*>)/g, function(match, contents, offset, s) {
 						if (variablesParsed[contents] !== undefined) {
 							return $.pkp.classes.TinyMCEHelper.prototype.getVariableElement(
-									contents, variablesParsed[contents]).html();
+									contents, variablesParsed[contents], '#' + tinyMCEObject.id)
+									.html();
 						}
 						return match;
 					});
@@ -413,16 +461,18 @@
 
 
 	/**
-	 * Fetch the header (e.g. on header configuration change).
+	 * Reload a tab.
 	 * @private
 	 * @param {HTMLElement} sourceElement The element that issued the
-	 *  update header event.
-	 * @param {Event} event The "fetch header" event.
+	 *  "reloadTab" event.
+	 * @param {Event} event The "reload tab" event.
+	 * @param {Object} jsonData The JSON content representing the
+	 *  reload request.
 	 */
-	$.pkp.controllers.SiteHandler.prototype.updateHeaderHandler_ =
-			function(sourceElement, event) {
-		var handler = $.pkp.classes.Handler.getHandler($('#navigationUserWrapper'));
-		handler.reload();
+	$.pkp.controllers.SiteHandler.prototype.reloadTabHandler_ =
+			function(sourceElement, event, jsonData) {
+
+		$(jsonData.tabsSelector).tabs('load', jsonData.tabSelector);
 	};
 
 
@@ -436,14 +486,11 @@
 	 * @param {Event} event The "call when click outside" event.
 	 * @param {{
 	 *   container: jQueryObject,
-	 *   callback: Function,
-	 *   skipWhenVisibleModals: boolean
+	 *   callback: Function
 	 *   }} eventParams The event parameters.
 	 * - container: a jQuery element to be used to test if user click
 	 * outside of it or not.
 	 * - callback: a callback function in case test is true.
-	 * - skipWhenVisibleModals: boolean flag to tell whether skip the
-	 * callback when modals are visible or not.
 	 */
 	$.pkp.controllers.SiteHandler.prototype.callWhenClickOutsideHandler_ =
 			function(sourceElement, event, eventParams) {
@@ -452,12 +499,13 @@
 			return;
 		}
 
-		if (eventParams.callback === undefined) {
-			return;
-		}
-
 		var id = eventParams.container.attr('id');
-		this.outsideClickChecks_[id] = eventParams;
+
+		if (eventParams.clear) {
+			delete this.outsideClickChecks_[id];
+		} else if (eventParams.callback !== undefined) {
+			this.outsideClickChecks_[id] = eventParams;
+		}
 	};
 
 
@@ -490,7 +538,6 @@
 	 * option avoids it, use the callback.
 	 * @private
 	 * @param {{
-	 *   skipWhenVisibleModals: boolean,
 	 *   container: Object,
 	 *   callback: Function
 	 *   }} checkOptions Object with data to be used to
@@ -518,17 +565,6 @@
 			return false;
 		}
 
-		// Check for the visible modals option.
-		if (checkOptions.skipWhenVisibleModals !==
-				undefined) {
-			if (checkOptions.skipWhenVisibleModals) {
-				if (this.getHtmlElement().find('div.ui-dialog').length > 0) {
-					// Found a modal, return.
-					return false;
-				}
-			}
-		}
-
 		// Do the click origin checking.
 		if ($container.has(event.target).length === 0) {
 
@@ -573,7 +609,7 @@
 			}
 		}
 		if (unsavedElementCount > 0) {
-			return $.pkp.locale.form_dataHasChanged;
+			return pkp.localeKeys['form.dataHasChanged'];
 		}
 		return undefined;
 	};
@@ -616,7 +652,8 @@
 	 */
 	$.pkp.controllers.SiteHandler.prototype.showNotification_ =
 			function(jsonData) {
-		var workingJsonData, notificationsData, levelId, notificationId, pnotify;
+		var workingJsonData, notificationsData, levelId, notificationId,
+				addclass, type;
 
 		workingJsonData = this.handleJson(jsonData);
 		if (workingJsonData !== false) {
@@ -624,7 +661,16 @@
 				notificationsData = workingJsonData.content.general;
 				for (levelId in notificationsData) {
 					for (notificationId in notificationsData[levelId]) {
-						pnotify = new PNotify(notificationsData[levelId][notificationId]);
+						addclass = notificationsData[levelId][notificationId].addclass;
+						type = 'notice';
+						if (addclass == 'notifySuccess') {
+							type = 'success';
+						} else if (addclass == 'notifyWarning' || addclass == 'notifyError' ||
+								addclass == 'notifyFormError' || addclass == 'notifyForbidden') {
+							type = 'warning';
+						}
+						pkp.eventBus.$emit('notify',
+								notificationsData[levelId][notificationId].text, type);
 					}
 				}
 			}
@@ -700,10 +746,9 @@
 	 */
 	$.pkp.controllers.SiteHandler.prototype.unregisterScrollingObserver_ =
 			function(siteHandler, siteHandlerElement, event, observerFunction) {
-		var castObserverFunction = /** @type {function()} */ observerFunction;
+		var castObserverFunction = /** @type {function()} */ (observerFunction);
 		$(document).unbind('scroll', castObserverFunction);
 		return false;
 	};
 
-/** @param {jQuery} $ jQuery closure. */
 }(jQuery));
